/* Image mirroring program with selectable line of horizontal symmetry 
    by Walfas/KirbyM ( http://ee.walfas.org/ ) 
    
    Note: Doesn't work as well for really really wide images 
*/

// Initialize images
PImage img0;  // Original image
PImage img1;  // Mirrored image #1
PImage img2;  // Mirrored image #2
PImage imgZ;  // Small part of original image with symmetry line on top
PImage imgZ2; // Image to store zoomed-in box

// Variables for storing filename and extension
String fileLocation, fileName, fileType;
String[] slashSplit = {};
String[] dotSplit = {};

// Size of window relative to height of screen
float windowSize = .9;

// Scaled-down image dimensions
int sImgWidth, sImgHeight;
int sImgWidth2a, sImgWidth2b, sImgHeight2;

// Variables to store the line of symmetry x-value
int symmetryX, screenSymmetryX;

// Zoom box variables
int screenZoomBoxX, screenZoomBoxY;  // Position on screen
int zoomBoxX, zoomBoxY;  // Position on full-sized original image
int zoomSize = 40;  // Box size on full-sized original image
int zoomBoxSize = 50;  // Box size on screen
int zoomBoxSizeMin = 25;  // Min box size on screen
int zoomBoxSizeMax = 150; // Max box size on screen

// Colors, fonts, etc
int bgColor = #FFFFFF;
int lineColor = #FF0000;
int textColor = #000000;

PFont font;
String fontName = "Arial";
int fontSize = 15;

void setup() {
  font = createFont(fontName, fontSize);  // Initialize font
  textFont(font);
  
  // Determine image location, file name, and extension
  fileLocation = selectInput();
  slashSplit = split(fileLocation,'/');
  
  fileName = slashSplit[slashSplit.length-1];
  dotSplit = split(fileName, '.');
  fileType = dotSplit[dotSplit.length-1];
  fileName = dotSplit[0];

  // Load unaltered image into img0
  img0 = loadImage(fileLocation);
  imgZ = createImage(img0.width,img0.height,RGB);
  imgZ2 = createImage(zoomBoxSize,zoomBoxSize,RGB);
  
  // Determine size of window based on dimensions of image
  // If the image is smaller than half the screen height, don't scale it
  if (img0.height < int(screen.height*windowSize/2))
    sImgHeight = img0.height;
  else
    sImgHeight = int(screen.height*windowSize/2);
  // Scale the width proportionally
  sImgWidth = int(map(img0.width, 0, img0.height, 0, sImgHeight));

  // Initialize the window with these specified dimensions
  size(sImgWidth, sImgHeight*2+fontSize*2);
  
  // Dimensions of scaled output images on screen
  sImgHeight2 = sImgHeight/2;
  
  symmetryX = img0.width/2;
  screenSymmetryX = width/2;
  updateImgZ();
  refreshImages();
}

void draw() {
  if (mousePressed && mouseY < sImgHeight && mouseX >= 0 && mouseX < width) {
    if (mouseButton == LEFT) {
      refreshImages();
      symmetryX = int(map(mouseX,0,width,0,img0.width));
      screenSymmetryX = mouseX;
      updateImgZ();
    }
    else if (mouseButton == RIGHT) {
      screenZoomBoxX = mouseX;
      screenZoomBoxY = mouseY;
      zoomBoxX = int(map(mouseX,0,width,0,img0.width));
      zoomBoxY = int(map(mouseY,0,sImgHeight,0,img0.height));
      refreshImages();
    }
  }
}

// Update the PImage "imgZ" with a vertical line
void updateImgZ() {
  imgZ.loadPixels();
  imgZ.copy(img0,zoomBoxX-zoomSize/2,zoomBoxY-zoomSize/2,zoomSize+1,zoomSize+1,
              zoomBoxX-zoomSize/2,zoomBoxY-zoomSize/2,zoomSize+1,zoomSize+1);
  for (int i=0; i<imgZ.height; i++)
    imgZ.pixels[imgZ.width*i+symmetryX] = color(lineColor);
  imgZ.updatePixels();
}

// Refresh the zoom box, updating position and size
void updateZoomBox() {
  noFill();
  stroke(textColor);
  rectMode(CENTER);
  updateImgZ();
  imgZ2 = createImage(zoomBoxSize,zoomBoxSize,RGB);
  imgZ2.copy(imgZ,zoomBoxX-zoomSize/2,zoomBoxY-zoomSize/2,zoomSize,zoomSize,
              0,0,zoomBoxSize,zoomBoxSize);
  image(imgZ2,screenZoomBoxX-zoomBoxSize/2,screenZoomBoxY-zoomBoxSize/2,zoomBoxSize,zoomBoxSize);
  rect(screenZoomBoxX,screenZoomBoxY,zoomBoxSize,zoomBoxSize);
}

// Checks for keys pressed
void keyPressed() {
  if (key == CODED) {
    switch(keyCode) {
      // Right and left are used to nudge the symmetry line
      case RIGHT:
        symmetryX = constrain(symmetryX+1,0,img0.width);
        screenSymmetryX = int(map(symmetryX,0,img0.width,0,width));
        updateImgZ();
        refreshImages();
        break;
      case LEFT:
        symmetryX = constrain(symmetryX-1,0,img0.width);
        screenSymmetryX = int(map(symmetryX,0,img0.width,0,width));
        updateImgZ();
        refreshImages();
        break;
        
      // Up and down are used to alter the size of the zoom box
      case UP:
        zoomBoxSize = constrain(zoomBoxSize+1,zoomBoxSizeMin,zoomBoxSizeMax);
        refreshImages();
        break;
      case DOWN:
        zoomBoxSize = constrain(zoomBoxSize-1,zoomBoxSizeMin,zoomBoxSizeMax);
        refreshImages();
        break;
    }
  }
  else {
    switch(key) {
      // Use the S key to save
      case 's':
        img1.save(fileName+"_1."+fileType);
        img2.save(fileName+"_2."+fileType);
        textAlign(RIGHT,BOTTOM);
        fill(bgColor);
        noStroke();
        rectMode(CORNER);
        rect(0,height-fontSize,width,fontSize);
        fill(textColor);
        text("Saved "+nf(hour(),2)+":"+nf(minute(),2)+":"+nf(second(),2),width,height);
        break;
    }
  }
}

void refreshImages() {
  background(bgColor);
  stroke(lineColor);
  image(img0,0,0,sImgWidth,sImgHeight);
  line(screenSymmetryX,0,screenSymmetryX,sImgHeight);
  
  fill(textColor);
  textAlign(LEFT,BOTTOM);
  text("Press S to save",0,height);
  text("← →: Nudge",0,height-fontSize);
  textAlign(RIGHT,BOTTOM);
  text("↑ ↓ R-Click: zoom",width,height-fontSize);
  
  img1 = mirrorImage(true);
  sImgWidth2a = int(map(img1.width,0,img0.width*2,0,sImgWidth));
  image(img1,width/2-sImgWidth2a/2,sImgHeight+1,sImgWidth2a,sImgHeight2);
  
  img2 = mirrorImage(false);
  sImgWidth2b = int(map(img2.width,0,img0.width*2,0,sImgWidth));
  image(img2,width/2-sImgWidth2b/2,sImgHeight+sImgHeight2+1,sImgWidth2b,sImgHeight2);
  
  updateZoomBox();
}

// Output a mirrored PImage
PImage mirrorImage(boolean leftSide) {
  PImage output;
  
  if (leftSide) {
    output = createImage(symmetryX*2, img0.height, RGB);
    output.loadPixels();
    for (int y=0; y<output.height; y++) {
      for (int x=0; x<symmetryX; x++) {
        output.pixels[y*output.width+x] = img0.pixels[y*img0.width+x];
        output.pixels[(y+1)*output.width-x-1] = output.pixels[y*output.width+x];
      }
    }
    
  }
  
  else {
    output = createImage((img0.width-symmetryX)*2, img0.height, RGB);
    output.loadPixels();
    for (int y=0; y<output.height; y++) {
      for (int x=0; x<img0.width-symmetryX; x++) {
        output.pixels[y*output.width+x+(img0.width-symmetryX)] = img0.pixels[y*img0.width+symmetryX+x];
        output.pixels[y*output.width+x] = img0.pixels[(y+1)*img0.width-x-1];
      }
    }
    
  }
  
  output.updatePixels();
  return output;
}
